#include "online.h"
#include "extensions.h"
#include <curses.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <signal.h>
#include <netinet/in.h>
#include <netdb.h>
#include <ctype.h>

#define YLINES 6

extern double calc_time();
extern int max_users;
extern char dhmbuf[];
extern char *version;
void display_userdata();
void display_userinfo();
void display_helpscreen();
void display_time();
void erase_middlebox();
void clear_window();
void enable_color();
void mvparse();

int currentuser=0, vpos=0, hpos=1, numlines=3, max=0;
int max_y, max_x;

double upload_speed, download_speed, total_speed;
unsigned int upload_num, download_num, total_num;

static WINDOW *infowin = NULL;
static WINDOW *mainmenu = NULL;
static int delay = 900000;
static int display_uinfo = 0;  /* Display user info box. */
static int display_help = 0;

#define STYLE_NORMAL    1
#define STYLE_GREEN     2
#define STYLE_BLUE      3
#define STYLE_RED       4

mainMenu()
{
 int ch;
#ifndef LINUX
 int retval;
 struct timeval tv;
 fd_set rfds;
#endif
/*-----------------------------------------------------------------------
SCREEN INITIALIZATION
-------------------------------------------------------------------------*/
screen_init:
 initscr();
 cbreak();
 nonl();
 intrflush(stdscr, FALSE);
 keypad(stdscr, TRUE);
 clear();
 noecho();
 curs_set(0);
/*-------------------------------------------------------------------------
COLOR INITIALIZATION
--------------------------------------------------------------------------*/
 start_color();
 init_pair(STYLE_NORMAL, COLOR_CYAN, COLOR_BLACK);
 init_pair(STYLE_GREEN, COLOR_BLACK, COLOR_CYAN);
 init_pair(STYLE_BLUE, COLOR_BLUE, COLOR_BLACK);
 init_pair(STYLE_RED, COLOR_RED, COLOR_BLACK);
/*------------------------------------------------------------------------*/

 mainmenu=newwin(0, 0, 0, 0);
 getmaxyx(mainmenu, max_y, max_x);	
 delwin(mainmenu);

 mainmenu=newwin(max_y, max_x, 0, 0);
 wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 wborder(mainmenu, 0,0,0,0,0,0,0,0);
 wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 mvwprintw(mainmenu, 1, 2, "glftpd spy %s", version);
 mvwprintw(mainmenu, 1, (max_x/4), "Refresh(%02d/32)", 33-(int)delay/100000);
 wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
 wattron(mainmenu, A_BOLD);
 mvwhline(mainmenu, 2, 1, 0, max_x - 2);
 mvwvline(mainmenu, 3, 13, 0, max_y - YLINES);
 mvwhline(mainmenu, max_y - YLINES + 3, 1, 0, max_x -2); 
 wattroff(mainmenu, A_BOLD);
 leaveok(mainmenu, TRUE);
 mvparse(max);

 while(1) {
#ifdef LINUX
   halfdelay(delay/100000);
   untouchwin(stdscr);
   ch = getch();
#else
   tv.tv_sec = 0;
   tv.tv_usec = delay;
   FD_ZERO(&rfds);
   FD_SET(0, &rfds);
   retval=select(1, &rfds, NULL, NULL, &tv);
   if (retval) 
	ch = getch();
   else
	ch = ERR;
#endif
   untouchwin(stdscr);
   switch(ch) {	
     case ERR:
	mvparse(max);
	break;

     case 13:
	if (display_uinfo) {
		erase_middlebox();
		display_uinfo = 0;
	} else {
		if (display_help) display_help = 0;
		display_uinfo = 1;
		if (online[currentuser].username[0] != '\0')
	        	display_userinfo();
	}
	break;

     case 'h': case 'H': case '?':
	if (display_help) {
		erase_middlebox();
		display_help = 0;
	} else {
		if (display_uinfo) display_uinfo = 0;
		display_help = 1;
		display_helpscreen();
	}
	break;

     case ',':
     case '<': delay += 400000;
     case '-':
     case '_':
      delay += 100000;
      if (delay >= 3200000)
	delay = 3200000;
      wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
      mvwprintw(mainmenu, 1, (max_x/4), "Refresh(%02d/32)", 33-(int)delay/100000);
      wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
      break;

     case '.':
     case '>': delay -= 400000;
     case '+':
     case '=':
      delay -= 100000;
      if (delay <= 100000)
	delay = 100000;
      wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
      mvwprintw(mainmenu, 1, (max_x/4), "Refresh(%02d/32)", 33-(int)delay/100000);
      wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
      break;

     case KEY_UP:
      vpos--;
      mvparse(0);
      break;

     case KEY_DOWN:
      vpos++;
      mvparse(1);	
      break;

     case KEY_HOME:
      max = 0;
      vpos = 0;
      mvparse(max);
      break;
   
     case 'k':
     case 'K':
      if (geteuid() == 0 && online[currentuser].procid > 0) {
	kill(online[currentuser].procid, SIGTERM);
      }
      mvparse(max);
      break;
   
     case 'r':
     case 'R':
      if (display_uinfo)
		erase_middlebox();
      clear();
      refresh();
      delwin(mainmenu);
      endwin();
      goto screen_init;
      break;
 
     case 'q': case 27:
      clear();
      refresh();
      def_prog_mode();
      endwin();
      return;
      break;

     default:
        break;
    }
  }
}
/*-----------------------------------------------------------------------*/
void mvparse(int pos) {
	int user = 0;
	int num_online = 0;
	int length, col;
	char str_online[50];
	char str_totalspeed[128];

	/* Fix me sometime for I am ugly!  */
	if (max_users < (max_y - YLINES)) {
		max = 0;
		if (vpos < 0) vpos = 0;
		if (vpos > max_users - 1) 
			vpos = max_users - 1;
	} else {
		if (vpos > (max_y - (YLINES+1))) {
			max++;
			vpos--;
			if (max > max_users - (max_y - YLINES))
				max--;
		} 
		if (vpos < 0) {
			vpos = 0;
			max--;
			if (max < 0) max = 0;
		}
		user += max;
	}

	for (col=0; col < max_users; col++) 
		if (online[col].procid != 0) num_online++;
		
        upload_speed = (double) 0.0; download_speed = (double) 0.0; total_speed = (double) 0.0;
        upload_num = 0;   download_num = 0;   total_num = 0;

 	for (col=0; user<max_users && col < (max_y - YLINES); user++, col++) {
		clear_window(numlines+col);
  		mvwprintw(mainmenu, numlines+col, 1, "            ");
		if (user == vpos+max) {
			wattron(mainmenu,A_REVERSE);
			mvwprintw(mainmenu, numlines+col, 1, "%-12.12s", online[user].username);
			wattroff(mainmenu,A_REVERSE);
			if (online[user].procid != 0) {
				currentuser = user;
				display_userdata(numlines+col, user);
			} 
		} 
		else if (online[user].procid != 0) {
			mvwprintw(mainmenu, numlines+col, 1, "%-12.12s", online[user].username);
			display_userdata(numlines+col, user);
		}
	}
 
	length = sprintf(str_online, "Users Online [%02d/%02d]", num_online, max_users);
 	wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	mvwprintw(mainmenu, 1, ((3*max_x/4) - (length/2)-5), str_online);

	sprintf (str_totalspeed, "Total Up: %7.1f K/s", upload_speed);
	mvwprintw(mainmenu, max_y - YLINES + 4, 2, str_totalspeed);

	length = sprintf (str_totalspeed, "Total Down: %7.1f K/s", download_speed);
	mvwprintw(mainmenu, max_y - YLINES + 4, ((max_x/2) - (length/2)-3), str_totalspeed);

        length = sprintf (str_totalspeed, "Total Bandwidth: %7.1f K/s", total_speed);
        mvwprintw(mainmenu, max_y - YLINES + 4, (max_x - length - 2), str_totalspeed);

	wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));

	display_time();
	display_userinfo(); /* Display info box if it is still up. */
	display_helpscreen();
	wrefresh(mainmenu);
}

/*------------------------------------------------------------------------*/
void display_userdata(int row, int user) {
   char action[BUFSIZ];
   char filename[100];
      
   memset(action, '\0', sizeof(action));
   memset(filename, '\0', sizeof(filename));
   strncpy(action, online[user].status, sizeof(action));
   if (action[strlen(action)-2] == '\r')
     action[strlen(action)-2] = '\0';

   if ((!strncasecmp(action, "STOR", 4) ||
       !strncasecmp(action, "APPE", 4)) && online[user].bytes_xfer != 0) {
   	if (strrchr(action+5, '/') != NULL) 
		strncpy(filename, strrchr(action+5, '/') + 1, sizeof(filename));
	else
		strncpy(filename, action+5, sizeof(filename));
      	snprintf(action, max_x - 15 , "UL: %luKB @ %.1fK/s, %-s", (online[user].bytes_xfer / 1024L), (double)calc_time(user), filename);
	trim(action);

	upload_speed += (double) calc_time(user);
	total_speed  += (double) calc_time(user);
	upload_num++;
	total_num++;

	mvwprintw(mainmenu, row, 15, action);
	wattron(mainmenu, A_BOLD);
	wattron(mainmenu, COLOR_PAIR(STYLE_RED));
	mvwaddch(mainmenu, row, 15, 'U');
	mvwaddch(mainmenu, row, 16, 'L');
	wattroff(mainmenu, COLOR_PAIR(STYLE_RED));
	wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	mvwaddch(mainmenu, row, 17, ':');
	wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	wattroff(mainmenu, A_BOLD);
   } else if (strncasecmp(action, "RETR", 4) == 0 && online[user].bytes_xfer != 0) {
   	if (strrchr(action+5, '/') != NULL)
		strncpy(filename, strrchr(action+5, '/') + 1, sizeof(filename));
	else
		strncpy(filename, action+5, sizeof(filename));
	filename[sizeof(filename)-1] = '\0';
	bzero(action, sizeof(action));
   	snprintf(action, max_x - 15, "DL: %luKB @ %.1fK/s, %s", (online[user].bytes_xfer / 1024L), (double)calc_time(user), filename);
	trim(action);

        download_speed += (double) calc_time(user);
        total_speed  += (double) calc_time(user);
        download_num++;
        total_num++;

	mvwprintw(mainmenu, row, 15, action);
	wattron(mainmenu, A_BOLD);
        wattron(mainmenu, COLOR_PAIR(STYLE_BLUE));
	mvwaddch(mainmenu, row, 15, 'D');
	mvwaddch(mainmenu, row, 16, 'L');
	wattroff(mainmenu, COLOR_PAIR(STYLE_BLUE));
	wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	mvwaddch(mainmenu, row, 17, ':');
	wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
	wattroff(mainmenu, A_BOLD);
   } else if ( (time(NULL) - online[user].tstart.tv_sec) > 5 ) {
   	hms_format(online[user].tstart.tv_sec);
	trim(dhmbuf);
	snprintf(action, max_x -1, "IDLE for %-s", dhmbuf);
   	trim(action);
	mvwprintw(mainmenu, row, 15, action);
   } else {
	trim(action);
   	mvwprintw(mainmenu, row, 15, "%-64.64s", action);
   }
}

/*-----------------------------------------------------------------------*/
void clear_window(int row)
{
	register int x;

	for (x=15; x < (max_x - 1); x++)
		mvwprintw(mainmenu, row, x, " ");
}

/*------------------------------------------------------------------------*/
void
display_userinfo(void)
{			   
	if (display_uinfo == 0)
		return;
	
	if (!infowin) {
 		infowin=subwin(mainmenu, 9, 60, max_y/2 - YLINES, max_x/2 - 30);
		touchwin(stdscr);	 
	}   
	wbkgdset(infowin, ' ' | COLOR_PAIR(STYLE_GREEN));
	werase(infowin);
	wbkgdset(infowin, ' ');
	wattrset(infowin, COLOR_PAIR(STYLE_GREEN));
 	box(infowin, 0, 0);
	wattron(infowin, COLOR_PAIR(STYLE_GREEN));
	mvwprintw(infowin, 1, 3, "User: %s", online[currentuser].username);
	mvwprintw(infowin, 2, 3, "TAG: %-46.46s", online[currentuser].tagline);
	mvwprintw(infowin, 3, 3, "PiD: %ld", online[currentuser].procid);
	mvwprintw(infowin, 4, 3, "CWD: %-51.51s", online[currentuser].currentdir);
	mvwprintw(infowin, 5, 3, "RHost: %-49.49s", online[currentuser].host);
	mvwprintw(infowin, 6, 3, "Login: %-49.49s", trim(ctime(&online[currentuser].login_time)));
	hms_format(online[currentuser].login_time);
	mvwprintw(infowin, 7, 3, "Time Online: %-s", trim(dhmbuf));
	wattrset(infowin, A_NORMAL);
	wnoutrefresh(infowin);
	doupdate();
}
void display_helpscreen(void) {			   
	if (display_help == 0)
		return;
	
	if (!infowin) {
 		infowin=subwin(mainmenu, 9, 60, max_y/2 - YLINES, max_x/2 - 30);
		touchwin(stdscr);	 
	}   
	wbkgdset(infowin, ' ' | COLOR_PAIR(STYLE_GREEN));
	werase(infowin);
	wbkgdset(infowin, ' ');
	wattrset(infowin, COLOR_PAIR(STYLE_GREEN));
 	box(infowin, 0, 0);
	wattron(infowin, COLOR_PAIR(STYLE_GREEN));
	mvwprintw(infowin, 1, 3, "+ , - : Increase/Decrease refresh rate by 1");
	mvwprintw(infowin, 2, 3, "> , < : Increase/Decrease refresh rate by 5");
	mvwprintw(infowin, 3, 3, "r     : Re-draw screen");
	mvwprintw(infowin, 4, 3, "Enter : Toggle detailed user view");
	mvwprintw(infowin, 5, 3, "h     : Toggle this Help screen");
	mvwprintw(infowin, 6, 3, "k     : Kick highlighted user (you must be root)");
	mvwprintw(infowin, 7, 3, "q     : Quit gl_spy");
	wattrset(infowin, A_NORMAL);
	wnoutrefresh(infowin);
	doupdate();
}
/*------------------------------------------------------------------------*/
void erase_middlebox(void) {
	werase(infowin);
	delwin(infowin);
	infowin=NULL;
	wattron(mainmenu, A_BOLD);
	mvwhline(mainmenu, 2, 1, 0, max_x - 2);
	mvwvline(mainmenu, 3, 13, 0, max_y - YLINES);
	wattroff(mainmenu, A_BOLD);
	wrefresh(mainmenu);
}
/**************************************************************************/
void display_time(void)
{
  time_t timebuff;
  struct tm *ptr;
  char currtime[10];

  timebuff = time(NULL);
  ptr = (struct tm *)localtime(&timebuff); 
  strftime(currtime, 10, "%H:%M:%S",ptr); 

  wattron(mainmenu, COLOR_PAIR(STYLE_NORMAL));
  mvwprintw(mainmenu, 1, max_x - 10, currtime);
  wattroff(mainmenu, COLOR_PAIR(STYLE_NORMAL));
} 
/**************************************************************************/
