#include "extensions.h"
#include "online.h"
#include <dirent.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <errno.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/ipc.h>
#include <sys/shm.h>

char *version = "v1.5b";

int max_users = 0;
char config_file[255];
char rootpath[MAXPATHLEN];
char datapath[MAXPATHLEN];
char dhmbuf[20];
void sysconfig();
void quit();
void init_shmem();
double calc_time(int pid);
char *get_grpid();

extern void init_scr(void);

static int shmid;
/**************************************************************************
	MAIN()   USEREDIT FUNCTION
***************************************************************************/
int main ( int argc, char **argv)
{
	if (argc > 1) {
		if (strncmp("-r", argv[1],2)) {
			printf("glFtpd Spy Utility %s\n\n", version);
			printf("Command line options:\n");  
			printf("        -r /etc/glftpd2.conf {use alternate config file}\n\n");
			printf("Help:   ESC or 'Q' to quit.\n");
			printf("        ENTER to display detailed information on user.\n");
			printf("        'k' to Kick User.  For ROOT only.\n");
			printf("	'r' to Refresh Display.\n");
			printf("        '+' & '-' to Alter Refresh Rate.\n");
			printf("        'h' or '?' for a help screen.\n");
			exit(0);
		} 
		else {
			if (argv[1][2]) 
				strncpy(config_file, argv[1]+2, sizeof(config_file));
			else
				strncpy(config_file, argv[2], sizeof(config_file));
			config_file[sizeof(config_file)-1] = '\0';
		}
	} else 	strcpy(config_file, "/etc/glftpd.conf");
	sysconfig();
	init_shmem();
	tzset();
	mainMenu();
	quit(0);
}
/*-----------------------------------------------------------------------*/
void sysconfig( void )
{
        struct aclmember *entry = NULL;

        readacl(config_file);
        getaclentry("rootpath", &entry);
        strncpy(rootpath, ARG0, sizeof(rootpath));
        entry = NULL;
        getaclentry("datapath", &entry);
        strncpy(datapath, ARG0, sizeof(datapath));
}
/*----------------------------------------------------------------------------*/
void init_shmem( void )
{
#define KEY     0x0000DEAD	
        struct aclmember *entry;
	key_t  key;

	entry = NULL;
	if (getaclentry("ipc_key", &entry) && ARG0) 
		key = strtoul(ARG0, (char **)NULL, 16);
	else
		key = KEY;		

        entry = NULL;
        getaclentry("max_users", &entry);
	if (!ARG0 || (max_users = atoi(ARG0)) < 1) {
		fprintf(stdout, "Max Users currently disabled!\n");
		exit(1);
	}
	if (ARG1 != NULL) max_users += atoi(ARG1);
	
	/* First try and creat the memory segment if it doesn't exist */
        if ((shmid = shmget( (key_t)key, (int)(sizeof(struct ONLINE) * max_users), IPC_CREAT | 0644 )) == -1) 
		/* Segment exist, use existing */
		/* Needed for non-root users apparently */
		if ((shmid = shmget( (key_t)key, (int)(sizeof(struct ONLINE) * max_users), 0)) == -1)
		{
        	       	fprintf(stdout, "Error: (SHMGET) Failed!\n");
                	exit(1);
	        }
        if ((online = (struct ONLINE *)shmat( shmid, NULL, SHM_RDONLY)) == (struct ONLINE *)-1)
	{
                fprintf(stdout, "Error: (SHMAT) Failed!\n");
                exit(1);
        }
}
/*-------------------------------------------------------------------------*/
void
quit(int exit_status)
{
        struct shmid_ds ipcbuf;

        shmctl( shmid, IPC_STAT, &ipcbuf);
        if (ipcbuf.shm_nattch <= 1) {
               shmctl( shmid, IPC_RMID, 0);
        }
        shmdt(0);
	exit(exit_status);
}
/*-------------------------------------------------------------------------*/

char *
trim( char *str )
{
	char *ibuf;
	char *obuf;
												 
   if ( str )
   {
   	for ( ibuf = obuf = str; *ibuf; )
      {
        	while ( *ibuf && ( isspace ( *ibuf ) ) )
         	ibuf++;
        	if ( *ibuf && ( obuf != str ) )
            *( obuf++ ) = ' ';
        	while ( *ibuf && ( !isspace ( *ibuf ) ) )
            *( obuf++ ) = *( ibuf++ );
      }
      *obuf = '\0';
   }
   return( str );
}
/*-- end of trim() -------------------------------------------------------*/

                              
void
hms_format( time_t dirtime )
{
   time_t timenow = time( NULL );
   time_t difftime;
   int    days = 0;
   int    hours = 0;
   int    minutes = 0;
   int	  seconds = 0;
   
   difftime = timenow - dirtime;
   
   while ( difftime >= (time_t)86400 )
   {
      difftime -= (time_t)86400;
      days++;
   }
   
   while ( difftime >= (time_t)3600 )
   {
      difftime -= (time_t)3600;
      hours++;
   }          
   
   while ( difftime >= (time_t)60 )
   {
      difftime -= (time_t)60;
      minutes++;
   }
   
   seconds = (int)difftime;

    if ( days != 0 )
       sprintf( dhmbuf, "%2dd %2dh", days, hours );
    else if ( hours != 0 )
       sprintf( dhmbuf, "%2dh %2dm", hours, minutes );
    else 
       sprintf( dhmbuf, "%2dm %2ds", minutes, seconds );
}
/*-- end of hms_format() -------------------------------------------------*/


double calc_time(int pid) {
	struct timeval tstop;
	double delta, rate;

	if (online[pid].bytes_xfer < 1)
		return 0;
        gettimeofday(&tstop, (struct timezone *)0 );
        delta = ((tstop.tv_sec*10.)+(tstop.tv_usec/100000.)) -
	        ((online[pid].tstart.tv_sec*10.)+(online[pid].tstart.tv_usec/100000.));
        delta = delta/10.;
        rate = ((online[pid].bytes_xfer / 1024.0) / (delta));
	if (!rate) rate++;
	return (double)(rate);
}

/*
char *
get_grpid(long grpid, char *grp_name)
{
        FILE *grp_file;
        char grp_path[MAXPATHLEN];
        char filebuff[BUFSIZ];
        char *tokptr;
        gid_t grp_gid;
        char grp_desc[BUFSIZ];
	int x;

        snprintf(grp_path, sizeof(grp_path), "%s/etc/group", rootpath);
        grp_file = fopen(grp_path, "r");
        if (grp_file == NULL) 
                return NULL;
        while (!feof(grp_file))
        {
                if (fgets(filebuff, 127, grp_file) == NULL && !feof(grp_file))                        
                        break;
                if (feof(grp_file))
                        break;
                
		for (x=0; x < strlen(filebuff); x++)
			if (filebuff[x] == '#')
				filebuff[x] = '\0';
	if (strlen(filebuff) < 1)
		continue;
		tokptr = strtok(filebuff, ":");
                if (tokptr)
                        strcpy(grp_name, tokptr);
                if (tokptr[strlen(tokptr)+1] != ':') {
			tokptr = strtok('\0', ":");
       		        if (tokptr)
       		                 strcpy(grp_desc, tokptr);
                }
		tokptr = strtok('\0', ":");
                if (tokptr) {
                        grp_gid = strtol(tokptr, (char **)NULL, 0);
			if (errno == ERANGE)
				continue;
		}
                if (grp_gid == grpid) {
                        fclose(grp_file);
                        return (grp_name);
                }
        }
        fclose(grp_file);
        return NULL;
}
*/
